@@ -1,6 +1,3 @@
-"""
-This file defines the QCLayer class for simulating QC structure
-"""
 import numpy as np
 from numpy import sqrt, pi, exp
 from numpy import fft
@@ -19,8 +16,8 @@ import copy
 from typing import List, Tuple, Union
 
 EUNIT = 1e-5    # E field unit from kV/cm to V/Angstrom
-BASISPAD = 100  # padding barrier for basis solver, unit Angstrom
-INV_INF = 1e-20  # for infinite small decay rate (ns-1)
+BASISPAD = 100  #: padding barrier for basis solver, unit Angstrom
+INV_INF = 1e-20  #: for infinite small decay rate (ns-1)
 # used for typing as either an array or a float number
 ScalerOrArray = Union[float, np.ndarray]
 
@@ -28,7 +25,7 @@ QCMaterial = {
     "InP":  ["InGaAs", "AlInAs"],
     "GaAs": ["AlGaAs"],
     "GaSb": ["InAsSb", "AlGaSb"]
-}
+}  #: supported substrate/material set
 
 
 class StateRecognizeError(Exception):
@@ -44,86 +41,87 @@ class StateRecognizeError(Exception):
 
 
 class SchrodingerLayer(object):
-    """Class for layer structure for Schrodinger solver using different
-    Eigen solver.
+    """Class for layer structure for Schrodinger solver.
 
     This is used as the base class of :class:`.QCLayers` for separation of
     material property and the solver.
 
-    Parameters
-    ----------
-    xres : float
-        Position resolution, in Armstrong
-    Eres : float
-        Energy resolution, in meV.
-        This number being too large may results in miss of some states while
-        this being too small will make a long computation time.
-        The parameter does not mean the accuracy of the eigen energy. It's
-        required for algorithm reasons because of lack of a universal global
-        root finding
-    statePerRepeat : int
-        Number of states per repeat, used for calculating matrixEigenCount
+Parameters
+----------
+xres :
+    Position resolution, in Armstrong
+Eres :
+    Energy resolution, in meV.
+    This number being too large may results in miss of some states while
+    this being too small will make a long computation time.
+    The parameter does not mean the accuracy of the eigen energy. It's
+    required for algorithm reasons because of lack of a universal global
+    root finding.
+statePerRepeat :
+    Number of states per repeat, used for calculating matrixEigenCount
 
-    layerWidths : list of float, len = No. of layers
-        Width of each layer, in unit angstrom
-    layerVc : list of float, len = No. of layers
-        The conduction band offset of each layer, in unit eV
-    layerMc : list of float, len = No. of layers
-        The conduction band effective mass of each layer, in unit m0
-        the free space electron mass
-    layerARs : list of bool, len = No. of layers
-        Binaries indicating if the layer is active(True) or not(False),
-        only affects basis solver
+layerWidths :
+    Width of each layer, in angstrom. len = No. of layers
+layerVc :
+    The conduction band offset of each layer, in unit eV, len = No. of layers
+layerMc :
+    The conduction band effective mass of each layer, in unit m0,
+    the free space electron mass, len = No. of layers
+layerARs :
+    Binaries indicating if the layer is active(True) or not(False),
+    only affects basis solver, len = No. of layers
 
-    ifrDelta : List of float, len = No. of layers
-        The standard deviation of the interface roughness for the interface at
-        layer n and layer n+1, in unit angstrom.
-        Default zero
-    ifrLambda : List of float, len = No. of layers
-        The correlation length of the interface roughness for the interface at
-        layer n and layer n+1, in unit angstrom.
-        Default zero
-    avghwLO : float
-        The average LO phonon energy in unit eV.
-        This value is used for LO phonon scattering calculation.
-    epsrho : float
-        The effective relative permittivity for LO phonon.
-        1/epsrho = 1/(epsilon for high frequency) - 1/(epsilon for static)
-        The value is used for LO phonon scattering calculation.
+ifrDelta :
+    The standard deviation of the interface roughness for the interface at
+    layer n and layer n+1, in unit angstrom, len = No. of layers.
+    Default zero
+ifrLambda :
+    The correlation length of the interface roughness for the interface at
+    layer n and layer n+1, in unit angstrom, len = No. of layers.
+    Default zero
+avghwLO :
+    The average LO phonon energy in unit eV.
+    This value is used for LO phonon scattering calculation.
+epsrho :
+    The effective relative permittivity for LO phonon.
+    1/epsrho = 1/(epsilon for high frequency) - 1/(epsilon for static)
+    The value is used for LO phonon scattering calculation.
 
-    EField : float
-        External (static) electrical field, in kV/cm = 1e5 V/m
-    repeats : int
-        Number of repeat times for the given structure
+EField :
+    External (static) electrical field, in kV/cm = 1e5 V/m
+repeats :
+    Number of repeat times for the given structure
 
-    crystalType : str
-        Default being "simple", meaning a simple parabolic effective mass is
-        used for the calculation. For setting other than "simple",
-        `populate_material` should be implemented.
+crystalType :
+    Default being "simple", meaning a simple parabolic effective mass is
+    used for the calculation. For setting other than "simple",
+    `populate_material` should be implemented.
 
-    basisAROnly : bool
-        For basis solver if only the Active Region (AR) should be solved.
-    basisARInjector : bool
-        For basis solver if there should be separator between AR->Injector
-    basisInjectorAR : bool
-        For basis solver if there should be separator between Injector->AR
+basisAROnly :
+    For basis solver if only the Active Region (AR) should be solved.
+basisARInjector :
+    For basis solver if there should be separator between AR->Injector
+basisInjectorAR :
+    For basis solver if there should be separator between Injector->AR
 
-    solver : str
-        The solver used for the eigen problem: 'ODE' or 'matrix'.
-        By default 'ODE' if C library exists, 'matrix' is a full back.
-    includeIFR : bool
-        Weather to include IFR scattering for performance estimation.
-    matrixEigenCount : int
-        The number of eigen pairs to calculate in the 'matrix' solver.
-        It would be very expensive to calculate all of them.
+solver :
+    The solver used for the eigen problem: 'ODE' or 'matrix'.
+    By default 'ODE' if C library exists, 'matrix' is a full back.
+includeIFR :
+    Weather to include IFR scattering for performance estimation.
+matrixEigenCount :
+    The number of eigen pairs to calculate in the 'matrix' solver.
+    It would be very expensive to calculate all of them.
 
-    status : str
-        - "unsolved" meaning the structure is not solved yet.
-        - "solved" meaning the eigen problem is solved.
-        - "solved-full" meaning the population distribution is known.
+status :
+    - 'unsolved' meaning the structure is not solved yet.
+    - 'basis' meaning the eigen problem is solved for basis
+    - 'solved' meaning the eigen problem is solved.
+    - 'solved-full' meaning the population distribution is known.
     """
     xres: float
     Eres: float
+    statePerRepeat: int
     layerWidths: List[float]
     _layerVc: List[float]
     _layerMc: List[float]
@@ -243,10 +241,12 @@ class SchrodingerLayer(object):
         self.periodL = layerCumSum[-1]
         self.xPoints = np.arange(0, self.periodL*self.repeats, self.xres)
         N = self.xPoints.size
+        if N == 0:
+            self.xPoints = np.array([0])
+            N = 1
         self.xLayerNums = np.empty(N, dtype=int)
         self.xVc = np.empty(N)
         self.xMc = np.empty(N)
-        self.xARs = np.empty(N, dtype=bool)
 
         for n in range(len(self.layerWidths)):
             indices = np.logical_or.reduce([
@@ -256,11 +256,10 @@ class SchrodingerLayer(object):
             self.xLayerNums[indices] = n
             self.xVc[indices] = self.layerVc(n)
             self.xMc[indices] = self.layerMc(n)
-            self.xARs[indices] = self.layerARs[n]
         if self.crystalType != 'simple':
             self.populate_material()
 
-        self.offset = max(self.xVc) - min(self.xVc)
+        self.offset = np.max(self.xVc) - np.min(self.xVc)
         self.xVField = self.xPoints * self.EField * EUNIT
         self.xVc -= self.xVField
 
@@ -576,7 +575,7 @@ class SchrodingerLayer(object):
             self.eigenEs = np.concatenate((self.eigenEs, eigenEs))
             self.psis = np.concatenate((self.psis, psis))
         self._reset_cache()
-        self.status = 'solved'
+        self.status = 'basis'
         return self.eigenEs
 
     def _reset_for_basis(self, start: int, end: int):
@@ -873,6 +872,7 @@ class SchrodingerLayer(object):
         unBound : set of int
             includes index of states that are not well bounded.
         """
+        # TODO: try look from the low energy side
         periodIdx = self.periodL / self.xres
         psisq = np.abs(self.psis)**2
         self.starts = np.argmax(psisq > tol, axis=1)
@@ -948,6 +948,7 @@ class SchrodingerLayer(object):
         flow : float
             The flow of carrier, in unit ps^-1 (carrier density normalize to 1)
         """
+        assert(self.status.startswith('solved'))
         idxPeriod = len(self.periodIdx)
         # p for cache for the periodic version
         self._pLO = [np.zeros((idxPeriod, idxPeriod)) for _ in range(3)]
@@ -1031,52 +1032,86 @@ substrate : str
     GaSb      InAs\ :sub:`y`\ Sb\ :sub:`1-y`   Al\ :sub:`x`\ Ga\ :sub:`1-x`\ Sb
     ========= ================================ ================================
 
-materials : list of str, len >= 2
-    Name of alloys
-moleFrac : list of float, len = Mp. of materials
-    mole fraction for each possible layer material,
-xres : float
+materials :
+    Name of alloys for the heterostructure materials, len >= 2
+moleFracs :
+    mole fraction for each possible layer material, len = Mp. of materials
+xres :
     Position resolution, in Armstrong
-Eres : float
+Eres :
     Energy resolution, in meV.
     This number being too large may results in miss of some states while
     this being too small will make a long computation time.
     The parameter does not mean the accuracy of the eigen energy. It's
     required for algorithm reasons because of lack of a universal global
     root finding.
-wl : float
-    The wavelength for the design, in unit um, for book keeping and
-    optimization, but doesn't go into quantum solver
+statePerRepeat :
+    Number of states per repeat, used for calculating matrixEigenCount
+wl :
+    The wavelength for the design, in unit um, gain spectrum and optimization,
+    but doesn't go into quantum solver
 
-layerWidths : list of float, len = No. of layers
-    Width of each layer, in angstrom
-layerMtrls : list of int, len = No. of layers
-    Label of materials, depending on substrate
-layerDopings : list of float, len = No. of layers
-    Doping per volume in unit 1e17 cm-3
-layerARs : list of bool, len = No. of layers
+layerWidths :
+    Width of each layer, in angstrom. len = No. of layers
+layerMtrls :
+    Label of materials, depending on substrate. len = No. of layers
+layerDopings :
+    Doping per volume in unit 1e17 cm-3. len = No. of layers
+layerARs :
     Binaries indicating if the layer is active(True) or not(False),
-    only affects basis solver
+    only affects basis solver. len = No. of layers
 
-customIFR : bool
+customIFR :
     Wether to use a customized IFR parameter rather than a material determined
     parameter.
-mtrlIFRLambda : list of float, len = No. of materials
-    The interface roughness lambda after materials[n]
-mtrlIFRDelta : list of float, len = No. of materials
-    The interface roughness delta after materials[n]
+mtrlIFRLambda :
+    The interface roughness lambda after materials[n], len = No. of materials
+mtrlIFRDelta :
+    The interface roughness delta after materials[n], len = No. of materials
 
-EField : float
+EField :
     External (static) electrical field, in kV/cm = 1e5 V/m
-repeats : int
+repeats :
     Number of repeat times for the given structure
-T : float
+T :
     Temperature of the device, affecting material property
-Solver : str
-    Name of solver
-description : str
-    Description of the data
+
+basisAROnly :
+    For basis solver if only the Active Region (AR) should be solved.
+basisARInjector :
+    For basis solver if there should be separator between AR->Injector
+basisInjectorAR :
+    For basis solver if there should be separator between Injector->AR
+
+solver :
+    The solver used for the eigen problem: 'ODE' or 'matrix'.
+    By default 'ODE' if C library exists, 'matrix' is a full back.
+includeIFR :
+    Weather to include IFR scattering for performance estimation.
+matrixEigenCount :
+    The number of eigen pairs to calculate in the 'matrix' solver.
+    It would be very expensive to calculate all of them.
+
+status :
+    - 'unsolved' meaning the structure is not solved yet.
+    - 'basis' meaning the eigen problem is solved for basis
+    - 'solved' meaning the eigen problem is solved.
+    - 'solved-full' meaning the population distribution is known.
+
+description :
+    Description of the data. For book-keeping purposes.
     """
+    materials: List[str]
+    moleFracs: List[float]
+    wl: float
+    layerMtrls: List[int]
+    layerDopings: List[float]
+    customIFR: bool
+    mtrlIFRLambda: List[float]
+    mtrlIFRDelta: List[float]
+    T: float
+    description: str
+
     def __init__(self, substrate="InP", materials=["InGaAs", "AlInAs"],
                  moleFracs=[0.53, 0.52], xres=0.5, Eres=0.5, statePerRepeat=20,
                  layerWidths=[10.0], layerMtrls=None, layerDopings=None,
@@ -1120,7 +1155,7 @@ description : str
         self.solver = solver
         if onedq is None:
             self.solver = 'matrix'
-        self.update_strain()
+        self.update_mtrls()
 
     def __copy__(self):
         return QCLayers(
@@ -1154,15 +1189,25 @@ description : str
             ifrLambda = [0.0] * len(self.layerMtrls)
         return ifrDelta, ifrLambda
 
-    def update_strain(self):
-        """Update strain for the materials. This should be called every time
-        material parameters are updated."""
+    def update_mtrls(self):
+        """Update properties for the materials.
+        This should be called every time the material parameters are changed
+        directly via `materials`, `moleFracs` or `temperature`.
+
+        Yield
+        -----
+        a_parallel : float
+            The parallel crystal constant of the structure, determined by the
+            substrate material.
+
+        mtrlAlloys : List[Material.Alloy]
+            list of the Alloy objects for processing material properties.
+        """
         self.a_parallel = self.subM.param['alc']
         self.mtrlAlloys = [Material.Alloy(self.materials[idx],
                                           self.moleFracs[idx],
                                           self.temperature)
                            for idx in range(len(self.materials))]
-
         for al in self.mtrlAlloys:
             al.set_strain(self.a_parallel)
 
@@ -1179,19 +1224,25 @@ description : str
         self.mtrlAlloys[n] = Material.Alloy(mtrl, moleFrac, self.temperature)
         self.mtrlAlloys[n].set_strain(self.a_parallel)
 
-    def add_mtrl(self, mtrl: str = None, moleFrac: float = None):
+    def add_mtrl(self, mtrl: str = None, moleFrac: float = None,
+                 IFRLambda: float = None, IFRDelta: float = None):
         """Add a new material possibility"""
         self.materials.append(mtrl if mtrl else
                               QCMaterial[self.substrate][0])
         self.moleFracs.append(moleFrac if moleFrac else 0.0)
+        self.mtrlIFRLambda.append(IFRLambda if IFRLambda else
+                                  self.mtrlIFRLambda[-1])
+        self.mtrlIFRDelta.append(IFRDelta if IFRDelta else
+                                 self.mtrlIFRDelta[-1])
         self.mtrlAlloys.append(Material.Alloy(
             self.materials[-1], self.moleFracs[-1], self.temperature))
         self.mtrlAlloys[-1].set_strain(self.a_parallel)
 
     def del_mtrl(self, n: int):
-        """Delete materials labeled n. All layers of this material will
-        become previous materials[n-1 if n >0 else 1].  There should be
-        at least two materials otherwise there will be error"""
+        """Delete materials labeled n.
+        All layers of this material will become previous
+        `materials[n-1 if n >0 else 1]`.  There should be at least two
+        materials otherwise there will be error."""
         if len(self.materials) <= 2:
             raise ValueError("There should be at least 2 materials")
         self.materials.pop(n)
@@ -1233,14 +1284,14 @@ description : str
             self.crystalType = Material.MParam[subs]["Crystal"]
             matlN = len(self.materials)
             self.materials = (QCMaterial[subs]*matlN)[0:matlN]
-            self.update_strain()
+            self.update_mtrls()
         else:
             raise TypeError("Substrate %s not supported" % subs)
 
     def set_temperature(self, T: float):
         self.temperature = T
         self.subM.set_temperature(T)
-        self.update_strain()
+        self.update_mtrls()
 
     @property
     def mtrlOffset(self) -> float:
@@ -1335,8 +1386,7 @@ description : str
 
         Parameters
         ----------
-        upper : int
-        lower : int
+        upper, lower :
             define the transition from upper to lower
 
         Return
@@ -1559,6 +1609,3 @@ def optimize_global(qcl: QCLayers, iter: int = 50):
         if not changed:
             break
     print('Finished')
-
-
-# vim: ts=4 sw=4 sts=4 expandtab
